//get networks from window.performance for native
(function (window) {
  if (typeof AppInsightJSBridge == "undefined") return;

  //如果已经存在了的话，就返回，不用继续执行下去了
  if (window._appinsight_js_inj) return;

  window._appinsight_js_inj = {
    pageURL: "",
  };
  AppInsightJSBridge.logDebug("run only once");

  var isPerfmanceAvail = (function () {
    var isAvail;
    return function () {
      if (typeof isAvail === "boolean") return isAvail;
      try {
        var perf = window.performance;
        var timing = perf.timing || 1;
        var paint = perf.getEntriesByType("paint") || 1;
        isAvail = !!(timing && paint);
        return true;
      } catch (e) {
        isAvail = false;
        return false;
      }
    };
  })();

  function getWebViewTiming(entries) {
    if (!isPerfmanceAvail()) return;

    var resouceArray = [];

    for (var i = 0; i < entries.length; i++) {
      var a = entries[i];
      if (a.entryType === "navigation" || a.entryType === "resource") {
        if (a.responseEnd - a.startTime < 1) {
          continue;
        }
        if (a.fetchStart == 0) {
          a.fetchStart = a.startTime;
        }

        var urlName = a.name;
        var durationTime = a.duration;
        if (a.entryType === "navigation" && urlName === "") {
          urlName = document.URL;
        }

        if (a.entryType === "navigation") {
          durationTime =
            a.responseEnd - window.performance.timing.navigationStart;
          console.log(
            "_appinsight_js_inj.pageURL:" + _appinsight_js_inj.pageURL
          );
          if (_appinsight_js_inj.pageURL === urlName) {
            continue;
          } else {
            _appinsight_js_inj.pageURL = urlName;
          }
        }

        resouceArray.push({
          connectEnd: a.connectEnd,
          connectStart: a.connectStart,
          decodedBodySize: a.decodedBodySize,
          domainLookupEnd: a.domainLookupEnd,
          domainLookupStart: a.domainLookupStart,
          duration: a.duration,
          encodedBodySize: a.encodedBodySize,
          fetchStart: a.fetchStart,
          href: a.name,
          id: uuid,
          initiatorType: a.initiatorType,
          requestStart: a.requestStart,
          responseEnd: a.responseEnd,
          responseStart: a.responseStart,
          secureConnectionStart: a.secureConnectionStart,
          startTime: a.startTime,
          transferSize: a.transferSize,
        });
      }
    }

    var doc = document;
    var util = {
      each: function each(ary, func) {
        if (ary) {
          var i;
          for (i = 0; i < ary.length; i += 1) {
            if (ary[i] && func(ary[i], i, ary)) {
              break;
            }
          }
        }
      },
      uuid: function () {
        var d = Date.now();
        if (
          typeof performance !== "undefined" &&
          typeof performance.now === "function"
        ) {
          d += performance.now();
        }
        return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(
          /[xy]/g,
          function (c) {
            var r = (d + Math.random() * 16) % 16 | 0;
            d = Math.floor(d / 16);
            return (c === "x" ? r : (r & 0x3) | 0x8).toString(16);
          }
        );
      },
    };

    function now() {
      return Date.now ? Date.now() : new Date().valueOf();
    }

    var performance = window.performance
      ? window.performance
      : window.Performance;

    var uuid = util.uuid();

    function isHidden(elem) {
      var display = "none";
      if (window.getComputedStyle) {
        var view = elem.ownerDocument.defaultView;
        if (!view || !view.opener) {
          view = window;
        }
        display = view.getComputedStyle(elem).display;
      } else if (elem.currentStyle) {
        display = elem.currentStyle.display;
      }
      return display && display == "none";
    }

    function getOffsetTop(elem) {
      var box;
      if (typeof elem.getBoundingClientRect === "undefined") {
        return undefined;
      }
      box = elem.getBoundingClientRect();
      var pageYOffset = window.pageYOffset;
      return (
        box.top +
        (pageYOffset === 0 ? 0 : pageYOffset || doc.scrollTop || 0) -
        (doc.clientTop || 0)
      );
    }

    function getFirstScreenTime(fpTime, loadEventEndOffset) {
      var firstPaintTime = fpTime;
      var elem = document.createElement("img");
      if (!elem.getBoundingClientRect) {
        return firstPaintTime;
      }
      if (performance && performance.getEntriesByName) {
        var height = window.innerHeight;
        var imgs = [];
        util.each(doc.querySelectorAll("img"), function (item) {
          if (
            !isHidden(item) &&
            item.src != "" &&
            getOffsetTop(item) < height
          ) {
            imgs.push(item.src);
          }
        });
        util.each(imgs, function (item) {
          var resouceTiming = performance.getEntriesByName(item);
          if (resouceTiming.length) {
            var itemEndTime = resouceTiming[0].responseEnd;
            if (
              resouceTiming[0].fetchStart <= loadEventEndOffset &&
              itemEndTime > firstPaintTime
            ) {
              firstPaintTime = itemEndTime;
            }
          }
        });
      }
      return firstPaintTime;
    }

    function getFirstPaintTime() {
      var firstPaintTime = 0;
      if (performance) {
        var t = performance.timing;
        firstPaintTime = t.domLoading;
        if (performance.getEntriesByName) {
          util.each(
            doc.querySelectorAll("head>link, head>script"),
            function (item) {
              var name;
              if (item.tagName == "LINK") {
                name = item.href;
              } else if (
                item.tagName == "SCRIPT" &&
                !item.defer &&
                !item.async
              ) {
                name = item.src;
              }
              if (name) {
                var resouceTiming = performance.getEntriesByName(name);
                if (resouceTiming.length == 1) {
                  var itemEndTime =
                    resouceTiming[0].responseEnd + t.navigationStart;
                  if (itemEndTime > firstPaintTime) {
                    firstPaintTime = itemEndTime;
                  }
                }
              }
            }
          );
        }
        firstPaintTime = firstPaintTime - t.navigationStart;
      }
      return firstPaintTime;
    }

    var navigationTiming = window.performance.timing;
    var url = window.location.href;
    var timingArray = [];
    var firstPaint = 0;
    var firstContentPaint = 0;
    var performancePaintTiming = window.performance.getEntriesByType("paint");

    for (var i = 0; i < performancePaintTiming.length; ++i) {
      if (performancePaintTiming[i].entryType == "paint") {
        if (performancePaintTiming[i].name == "first-paint") {
          firstPaint = performancePaintTiming[i].startTime;
        } else if (performancePaintTiming[i].name == "first-contentful-paint") {
          firstContentPaint = performancePaintTiming[i].startTime;
        }
      }
    }

    if (firstPaint === 0) {
      firstPaint = getFirstPaintTime();
    }

    if (firstPaint === 0) {
      firstPaint =
        navigationTiming.responseStart - navigationTiming.navigationStart;
    }

    var firstScreenTime = getFirstScreenTime(
      firstPaint,
      navigationTiming.loadEventEnd
    );

    (function (addEventListener, removeEventListener) {
      var firstInputEvent;
      var firstInputDelay;
      var firstInputTimeStamp;
      var callbacks = [];
      var listenerOpts = { passive: true, capture: true };
      var startTimeStamp = new Date();
      var pointerup = "pointerup";
      var pointercancel = "pointercancel";

      function onFirstInputDelay(callback) {
        callbacks.push(callback);
        reportFirstInputDelayIfRecordedAndValid();
      }

      function recordFirstInputDelay(delay, evt) {
        if (!firstInputEvent) {
          firstInputEvent = evt;
          firstInputDelay = delay;
          firstInputTimeStamp = new Date();
          eachEventType(removeEventListener);
          reportFirstInputDelayIfRecordedAndValid();
        }
      }

      function reportFirstInputDelayIfRecordedAndValid() {
        if (
          firstInputDelay >= 0 &&
          firstInputDelay < firstInputTimeStamp - startTimeStamp
        ) {
          callbacks.forEach(function (callback) {
            callback(firstInputDelay, firstInputEvent);
          });
          callbacks = [];
        }
      }

      function onPointerDown(delay, evt) {
        function onPointerUp() {
          recordFirstInputDelay(delay, evt);
          removePointerEventListeners();
        }
        function onPointerCancel() {
          removePointerEventListeners();
        }
        function removePointerEventListeners() {
          removeEventListener(pointerup, onPointerUp, listenerOpts);
          removeEventListener(pointercancel, onPointerCancel, listenerOpts);
        }
        addEventListener(pointerup, onPointerUp, listenerOpts);
        addEventListener(pointercancel, onPointerCancel, listenerOpts);
      }

      function onInput(evt) {
        if (evt.cancelable) {
          var isEpochTime = evt.timeStamp > 1e12;
          var now = isEpochTime ? new Date() : performance.now();
          var delay = now - evt.timeStamp;
          if (evt.type == "pointerdown") {
            onPointerDown(delay, evt);
          } else {
            recordFirstInputDelay(delay, evt);
          }
        }
      }

      function eachEventType(callback) {
        var eventTypes = [
          "click",
          "mousedown",
          "keydown",
          "touchstart",
          "pointerdown",
        ];
        eventTypes.forEach(function (eventType) {
          callback(eventType, onInput, listenerOpts);
        });
      }
      eachEventType(addEventListener);
      self["perfMetrics"] = self["perfMetrics"] || {};
      self["perfMetrics"]["onFirstInputDelay"] = onFirstInputDelay;
    })(addEventListener, removeEventListener);

    perfMetrics.onFirstInputDelay(function (delay, evt) {
      var fidInfo = {};
      fidInfo.fid = delay;
      fidInfo.mpfid = 0;
      fidInfo.url = url;
      fidInfo.uuid = uuid;
      // window.webkit.messageHandlers.performance.postMessage({ fid: fidInfo });
      AppInsightJSBridge.logJsResultApm(
        JSON.stringify({
          fid: fidInfo,
        })
      );
    });

    timingArray.push({
      id: uuid,
      navigationStart: navigationTiming.navigationStart,
      redirectStart: navigationTiming.redirectStart,
      redirectEnd: navigationTiming.redirectEnd,
      connectEnd: navigationTiming.connectEnd,
      connectStart: navigationTiming.connectStart,
      domComplete: navigationTiming.domComplete,
      domContentLoadedEventEnd: navigationTiming.domContentLoadedEventEnd,
      domContentLoadedEventStart: navigationTiming.domContentLoadedEventStart,
      domInteractive: navigationTiming.domInteractive,
      domLoading: navigationTiming.domLoading,
      domainLookupEnd: navigationTiming.domainLookupEnd,
      domainLookupStart: navigationTiming.domainLookupStart,
      fetchStart: navigationTiming.fetchStart,
      loadEventEnd: navigationTiming.loadEventEnd,
      loadEventStart: navigationTiming.loadEventStart,
      navigationStart: navigationTiming.navigationStart,
      requestStart: navigationTiming.requestStart,
      responseEnd: navigationTiming.responseEnd,
      responseStart: navigationTiming.responseStart,
      secureConnectionStart: navigationTiming.secureConnectionStart,
      firstPaint: firstPaint,
      firstContentPaint: firstContentPaint,
      firstScreenTime: firstScreenTime,
    });

    AppInsightJSBridge.logJsResultApm(
      JSON.stringify({
        url: url,
        resource: resouceArray,
        timing: timingArray,
      })
    );
  }

  // js error 获取
  function jsErrorHandler(errorEvent) {
    if (!isPerfmanceAvail()) return;
    console.log(errorEvent);
    var err = errorEvent.error;
    if (!err) {
      return;
    }
    var navigationStart = window.performance.timing.navigationStart;
    var url = window.location.href;

    AppInsightJSBridge.logJsError(
      JSON.stringify({
        message: err.message,
        line: err.line,
        column: err.column,
        sourceURL: err.sourceURL,
        fileName: errorEvent.filename,
        url: url,
        stack: err.stack,
        navigationStart: navigationStart,
        offsetTimestamp: errorEvent.timeStamp,
        typeName: errorEvent.type,
      })
    );
  }
  window.addEventListener("error", jsErrorHandler, false);

  var isCallback = false;

  function insightObserverCallBack(list, obj) {
    var entries = list.getEntries();
    insightProcessData(entries);
  }

  function addObserverForPerformace() {
    if (!isCallback) {
      var elements = window.performance.getEntries();
      insightProcessData(elements);
      var observer = new PerformanceObserver(insightObserverCallBack);
      observer.observe({ entryTypes: ["navigation", "resource"] });
      isCallback = true;
    } else {
      AppInsightJSBridge.logDebug("已经回调过一次数据了,不再回调了");
    }
  }

  function insightAddOnLoadListener() {
    if (!isPerfmanceAvail()) return;
    window.addEventListener("load", addObserverForPerformace);
    window.addEventListener("beforeunload", addObserverForPerformace);
    window.addEventListener("pagehide", addObserverForPerformace);
    window.addEventListener("unload", addObserverForPerformace);
    setTimeout(addObserverForPerformace, 1000);
  }

  function insightProcessData(entries) {
    getWebViewTiming(entries);
  }

  insightAddOnLoadListener();
})(window);
